//
//  SITBuilding.h
//  SitumSDK
//
//  Created by A Barros on 1/3/17.
//  Copyright © 2017 Situm. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <CoreLocation/CoreLocation.h>

#import "SITAngle.h"
#import "SITDimensions.h"
#import "SITResource.h"
#import "SITURL.h"
#import "SITBounds.h"

/**
 Provides information about the places where indoor location is supported.
 */
@interface SITBuilding : SITResource

#pragma mark - Initializers

/** Initializer method
 @param createdAt Creation date
 @param updatedAt Last updated date
 @param customFields Key-value pairs that can be used to store information
 @param userIdentifier Identifier of the user who creates the building
 @param name Name if the building
 @param center Cartesian coordinates of the center of the building
 @param info String containing additional information
 @param dimensions Structure containing the widht and height of the building
 @param rotation Rotation angle of the building's base, relative to the west-east axis,
 increasing in counter-clockwise, being 0 the west-east axis.
 @param pictureURL URL of the building image
 @param pictureThumbURL URL of the thumbnail building image
 @return `SITBuilding`
 */
- (instancetype _Nonnull)initWithIdentifier:(NSString * _Nonnull)identifier
                                  createdAt:(NSDate * _Nonnull)createdAt
                                  updatedAt:(NSDate * _Nonnull)updatedAt
                               customFields:(NSDictionary * _Nonnull)customFields
                             userIdentifier:(NSString * _Nonnull)userIdentifier
                                       name:(NSString * _Nonnull)name
                                     center:(CLLocationCoordinate2D)center
                                       info:(NSString * _Nonnull)info
                                 dimensions:(SITDimensions * _Nonnull)dimensions
                                   rotation:(SITAngle * _Nonnull)rotation
                                 pictureURL:(SITURL * _Nonnull)pictureURL
                            pictureThumbURL:(SITURL * _Nonnull)pictureThumbURL;

#pragma mark - Properties

/**
 Identifier of the user who created the building
 */
@property(nonatomic, strong) NSString *  _Nonnull userIdentifier;

/**
 *  Name of the building.
 */
@property(nonatomic, strong) NSString *  _Nonnull name;

/**
 *  Address of the building.
 */
@property(nonatomic, strong) NSString * _Nonnull address;

/**
 *  Detailed information in HTML format
 */
@property(nonatomic, strong) NSString * _Nonnull infoHTML;

/**
 Rotation angle of the building's base, relative to the west-east axis,
 increasing in counter-clockwise, being 0 the west-east axis.
 */
@property(nonatomic, strong) SITAngle * _Nonnull rotation;

/**
 * URL of the building image
 */
@property(nonatomic, strong) SITURL * _Nonnull pictureURL;

/**
 * URL of the thumbnail building image
 */
@property(nonatomic, strong) SITURL * _Nonnull pictureThumbURL; // nail TODO: Complete names??

#pragma mark - Methods

/**
 * Compute corners of this building, without rotation, in earth coordinates.
 *
 * @return `SITBounds`
 */
- (SITBounds)bounds;

/**
 * Compute corners of this building, with rotation, in earth coordinates.
 *
 * @return `SITBounds`
 */
- (SITBounds)rotatedBounds;


/**
 Size of the building

 @return `SITDimensions` structure containing the widht and height of the building
 */
- (SITDimensions * _Nonnull)dimensions;

/**
 Geographical coordinate of the building
 
 @return `CLLocationCoordinate2D`
 */
- (CLLocationCoordinate2D)center;


@end
